#include <SPI.h>
#include <Ethernet.h>
#include "inkey.h"
#include "OneWire.h"
#include <avr/wdt.h>
#include "utility/w5100.h"

#define READ_TEMPERATURE 0
#define EXEC_CONVERSION 1

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(10, 210, 100, 1);
IPAddress myDns(10, 210, 1, 10);
IPAddress gateway(10, 210, 1, 1);
IPAddress subnet(255, 255, 0, 0);

EthernetServer server(23);

inkey extInput(A0);
inkey lineUPS(A1);
inkey lineA(A2);
inkey lineB(A3);

OneWire  ds(A4);

unsigned int thrmBuffer[2];
unsigned int thrmCounter = 1100;


// -- nastaveni --
void setup() {
  wdt_enable(WDTO_4S);
  Ethernet.begin(mac, ip, myDns, gateway, subnet);
  server.begin();
  Serial.begin(115200);
  Serial.print("IP address: ");
  Serial.println(Ethernet.localIP());
  readThrm(EXEC_CONVERSION);
  delay(1000);
  thrmBuffer[0] = readThrm(READ_TEMPERATURE);
  thrmBuffer[1] = thrmBuffer[0];
}


// -- pracovni smycka --
void loop() {
  unsigned char i = 0;
  byte remoteip[4];

  // -- obsluha sitoveho pripojeni --
  server.available();
  while (i < 4) {  
    EthernetClient client(i);
    if (client.connected()) {
      Serial.print("New connection from ");
      W5100.readSnDIPR(i, remoteip);
      Serial.println(IPAddress(remoteip));
      client.print("Arduino-Square-Box:");
      if (lineUPS._state) client.print("LineUPS-OFF|"); else client.print("LineUPS-ON|");
      if (lineA._state) client.print("LineA-OFF|"); else client.print("LineA-ON|");
      if (lineB._state) client.print("LineB-OFF|"); else client.print("LineB-ON|");
      if (extInput._state) client.print("ExtIn-OPEN|"); else client.print("ExtIn-CLOSE|");
      client.print("Thrm-");
      client.print((thrmBuffer[0] + thrmBuffer[1]) / 2, DEC);
      client.print("#");
      client.println();
      client.stop();
    }
    i++;
  }

  // -- obsluha smycek --
  if (extInput.beat()) {
    if (extInput._state) Serial.println("ExtIn OPEN"); else Serial.println("ExtIn CLOSE");
  }
  if (lineUPS.beat()) {
    if (lineUPS._state) Serial.println("LineUPS OFF"); else Serial.println("LineUPS ON");
  }
  if (lineA.beat()) {
    if (lineA._state) Serial.println("LineA OFF"); else Serial.println("LineA ON");
  }
  if (lineB.beat()) {
    if (lineB._state) Serial.println("LineB OFF"); else Serial.println("LineB ON");
  }

  // -- obsluha teplomeru --
  thrmCounter++;
  if (thrmCounter == 1180) readThrm(EXEC_CONVERSION);
  if (thrmCounter >= 1200) {
    thrmBuffer[1] = thrmBuffer[0];
    thrmBuffer[0] = readThrm(READ_TEMPERATURE);
    thrmCounter = 0;
    Serial.print("Read new temperature: ");
    Serial.println(thrmBuffer[0]);
  }

  delay(50);
  wdt_reset();
}


// -- spusti prevod nebo nacteni cidla --
unsigned int readThrm(unsigned char action) {
  unsigned char data_lo, data_hi;
  signed int t;
  unsigned int tout = 0;
  switch (action) {
    case EXEC_CONVERSION:
      ds.reset();
      ds.write(0xCC);
      ds.write(0x44);
      break;
    case READ_TEMPERATURE:
      ds.reset();
      ds.write(0xCC);
      ds.write(0xBE);
      data_lo = ds.read();
      data_hi = ds.read();
      t = ((data_hi << 8) | data_lo) * 0.625;
      tout = t + 1000;
      if ((tout < 500) || (tout > 1500)) tout = 0;
  }
  return (tout);
}






